indexing
	description: "Description of class"
	date: "$Date: Jan 14, 2007 9:54:30 AM$"
	revision: "$Revision: 1.0$"

class EM3D_MESH_OBJ_FACTORY

inherit
	EM3D_MESH_FACTORY

create
	make

feature {NONE} -- Initialization

	read_vertex( split_line: LIST[ STRING ] ): EM_VECTOR3D is
			-- Read a vector
		local
			i: INTEGER
		do
			from i:=1 until i>3 loop
				from until split_line.after or else split_line.item.is_double loop
					split_line.forth
				end
				if split_line.after then
					result[ i ] := 0
				else
					result[ i ] := split_line.item.to_double
				end
				i := i + 1
				split_line.forth
			end
		end
	read_triple( a_triple: STRING ): EM_VECTOR3I is
			-- Read a number triple ("a/b/c") from 'a_file'
		require
			a_triple_not_void: a_triple /= void
		local
			numbers: LIST[ STRING ]
			i: INTEGER
		do
			numbers := a_triple.split ('/')
			from i:=1; numbers.start until i>3 or numbers.after loop
				if numbers.item.is_integer then
					result[i] := numbers.item.to_integer
				end
				i:=i+1; numbers.forth
			end
			from until i>3 loop
				result[i] := 0
				i := i + 1
			end
		end
	read_string( split_line: LIST[ STRING ] ): STRING is
			-- Read a String
		do
			from until split_line.after or else not split_line.is_empty loop
				split_line.forth
			end
			if not split_line.after then
				result := split_line.item
			end
		end

	load_mesh_internal( a_filename: STRING ) is
			-- Load the mesh from an '.obj' file
		local
			file: PLAIN_TEXT_FILE
			command, line, mesh_name, mat_file: STRING
			split_line: LIST[ STRING ]
			v,n: ARRAY[ EM_VECTOR3D ]
			t: ARRAY[ EM_VECTOR2D ]
			i,j: INTEGER
			fi, li, ci: EM_VECTOR3I
			id: ARRAY[ EM_VECTOR3I ]
			mat: STRING
			mesh: EM3D_MESH
			tmp_meshes: ARRAY[ EM_PAIR[ STRING, EM3D_MESH ] ]
			material_list: ARRAY[ STRING ]
		do
			-- TODO: It would be really nice to have some error handling here

			mesh_name := a_filename

			-- Create the arrays with a given base size (we have enough memory)
			create v.make (1,0)
			create t.make (1,0)
			create n.make (1,0)
			create id.make(1,0)
			create file.make_open_read( a_filename )

			create material_list.make(0,-1)
			create tmp_meshes.make(0,-1)

			create mat.make_empty

			from until file.end_of_file loop
				-- Read line per line
				file.read_line
				line := file.last_string
				line.prune_all_trailing ( '%R' )

				-- Split the line up
				split_line := line.split ( ' ' )
				split_line.start
				command := split_line.item
				split_line.forth

				-- Parse the command
				if command.is_equal( "#" ) then -- We have a comment
				elseif command.is_equal( "v" ) then
					v.force( read_vertex(split_line), v.upper+1 )
				elseif command.is_equal( "vn" ) then
					n.force( read_vertex(split_line), n.upper+1 )
				elseif command.is_equal( "vt" ) then
					t.force( read_vertex(split_line), t.upper+1 )
				elseif command.is_equal( "mtllib" ) then
					mat_file := read_string( split_line )
					if file_system.file_exists ( mat_file ) then
						scene_manager.material_factory.load_material ( mat_file )
						material_list.force ( mat_file, material_list.upper+1 )
					else
						mat_file := file_system.pathname ( file_system.dirname ( a_filename ), mat_file)
						if file_system.file_exists ( mat_file ) then
							scene_manager.material_factory.load_material ( mat_file )
							material_list.force ( mat_file, material_list.upper+1 )
						else
							error_handler.raise_warning ( error_codes.em_error_em3d_mesh_factory, ["Material file '"+mat_file+"' not found!"] )
						end
					end

				elseif command.is_equal( "usemtl" ) then
					-- Create a new submesh containing the old material
					if mesh=void then
						create mesh.make ( scene_manager, t.count/=0 )
					end
					-- It look horrible, but I din't feel like creating two new local vars
					if t.count=0 then
						if n.count=0 then
							mesh.add_sub_mesh ( v, void, void, id, mat )
						else
							mesh.add_sub_mesh ( v, n, void, id, mat )
						end
					else
						if n.count=0 then
							mesh.add_sub_mesh ( v, void, t, id, mat )
						else
							mesh.add_sub_mesh ( v, n, t, id, mat )
						end
					end
					-- Set the new material and reset the indices
					mat := read_string( split_line )
					create id.make(1,0)
				elseif command.is_equal( "f" ) then
					-- We only support planar convex faces
					from i:=1 until split_line.after loop
						-- Skip the whitespace
						from until split_line.after or else not split_line.item.is_empty loop
							split_line.forth
						end
						ci := read_triple( split_line.item )
						-- We want VERTEX/NORMAL/TEX_COORD
						j := ci.y; ci.y := ci.z; ci.z := j
						if i = 1 then
							fi := ci
						elseif i > 2 then
							id.force (fi, id.upper+1)
							id.force (li, id.upper+1)
							id.force (ci, id.upper+1)
						end
						li := ci
						split_line.forth
						i := i + 1
					end
				end
			end
			if n.count = 0 then
				n := void
			end
			if t.count = 0 then
				t := void
			end
			if mesh=void then
				create mesh.make ( scene_manager, t/=void )
			end
			mesh.add_sub_mesh ( v, n, t, id, mat )
			add_mesh( mesh, mesh_name )

			tmp_meshes.force ( create {EM_PAIR[ STRING, EM3D_MESH ]}.make ( mesh_name, mesh ), tmp_meshes.upper+1 )
			scene_manager.mesh_factory.store_mesh ( a_filename+".mesh", tmp_meshes, material_list )
			file.close
		end


invariant
	the_invariant: True -- TODO: complete

end -- class MESH_OBJ_FACTORY
